import { splitAnnotationsAndCode } from "./extract-annotations.js";
import { isWhitespace, } from "./types.js";
import { highlight as lighter, LANG_NAMES, } from "@code-hike/lighter";
export async function highlight(data, theme, config = {}) {
    let { value, lang = "txt" } = data;
    if (!LANG_NAMES.includes(lang)) {
        console.warn(`Code Hike warning: Unknown language "${lang}"`);
        lang = "txt";
    }
    const { code, annotations } = await splitAnnotationsAndCode(value, lang, config.annotationPrefix || "!");
    const { lines, lang: lighterLang, style, } = await lighter(code, lang, theme, {
        annotations: [],
        scopes: false, // true for better token transitions, but breaks css themes
    });
    const tokens = joinLines(lines);
    // split surrounding whitespace for each token
    const splitTokens = splitWhitespace(tokens);
    // join consecutive whitespace tokens
    const joinedTokens = joinWhitespace(splitTokens);
    return {
        ...data,
        code,
        tokens: joinedTokens,
        lang: lighterLang,
        annotations: compatAnnotations(annotations),
        themeName: typeof theme === "string" ? theme : (theme === null || theme === void 0 ? void 0 : theme.name) || "unknown",
    };
}
function compatAnnotations(annotations) {
    const newAnnotations = [];
    for (const a of annotations) {
        const { name, query, ranges } = a;
        for (const r of ranges) {
            if (r.lineNumber) {
                const { lineNumber, fromColumn, toColumn } = r;
                newAnnotations.push({
                    name,
                    query,
                    lineNumber,
                    fromColumn,
                    toColumn,
                });
            }
            else {
                const { fromLineNumber, toLineNumber } = r;
                newAnnotations.push({
                    name,
                    query,
                    fromLineNumber,
                    toLineNumber,
                });
            }
        }
    }
    return newAnnotations;
}
// group the Lines into one array
function joinLines(lines) {
    const joinedTokens = [];
    lines.forEach((lineOrGroup, i) => {
        if ("lines" in lineOrGroup) {
            throw new Error("Shouldnt be groups");
        }
        else {
            const tokens = joinTokens(lineOrGroup.tokens);
            joinedTokens.push(...tokens);
            if (i < lines.length - 1) {
                joinedTokens.push("\n");
            }
        }
    });
    return joinedTokens;
}
function joinTokens(tokens) {
    return tokens.map((tokenOrGroup) => {
        if ("tokens" in tokenOrGroup) {
            throw new Error("Shouldnt be groups");
        }
        else {
            const t = [tokenOrGroup.content];
            const { color, ...rest } = tokenOrGroup.style || {};
            t.push(color);
            if (Object.keys(rest).length) {
                t.push(rest);
            }
            return t;
        }
    });
}
function splitWhitespace(tokens) {
    const ejected = [];
    tokens.forEach((tokenOrGroup) => {
        if (isWhitespace(tokenOrGroup)) {
            ejected.push(tokenOrGroup);
        }
        else {
            const [before, content, after] = splitSurroundingWhitespace(tokenOrGroup[0]);
            if (before === null || before === void 0 ? void 0 : before.length) {
                ejected.push(before);
            }
            if (content.length) {
                const copy = [...tokenOrGroup];
                copy[0] = content;
                ejected.push(copy);
            }
            if (after === null || after === void 0 ? void 0 : after.length) {
                ejected.push(after);
            }
        }
    });
    return ejected;
}
function joinWhitespace(tokens) {
    const joinedTokens = [];
    tokens.forEach((tokenOrGroup) => {
        if (isWhitespace(tokenOrGroup)) {
            let last = joinedTokens[joinedTokens.length - 1];
            if (last && isWhitespace(last)) {
                joinedTokens[joinedTokens.length - 1] += tokenOrGroup;
            }
            else if (tokenOrGroup !== "") {
                joinedTokens.push(tokenOrGroup);
            }
        }
        else if (tokenOrGroup[0].length > 0) {
            joinedTokens.push(tokenOrGroup);
        }
    });
    return joinedTokens;
}
// splits " \t foo bar \n" into [" \t ","foo bar"," \n"]
// "foo bar" -> ["","foo bar",""]
function splitSurroundingWhitespace(content) {
    const trimmed = content.trim();
    const before = content.slice(0, content.indexOf(trimmed));
    const after = content.slice(content.indexOf(trimmed) + trimmed.length);
    return [before, trimmed, after];
}
