import { jsx as _jsx } from "react/jsx-runtime";
import { forwardRef, useMemo } from "react";
import { RenderLineContent, toLineContent } from "./tokens.js";
import { isBlockAnnotation, isInlineAnnotation, } from "./types.js";
import { mergeProps } from "./merge-props.js";
import { getPreComponent } from "./component-reducer.js";
export const Pre = forwardRef(({ code, handlers = [], className, ...rest }, ref) => {
    let { tokens, themeName, lang, annotations } = code;
    handlers
        .filter((c) => c.transform)
        .forEach((c) => {
        annotations = annotations.flatMap((a) => c.name != a.name ? a : c.transform(a) || []);
    });
    if (!tokens) {
        throw new Error("Missing tokens in code block. Use the `highlight` function to generate the tokens.");
    }
    const lines = toLines(tokens);
    const indentations = lines.map((line) => { var _a, _b; return ((_b = (_a = line.tokens[0]) === null || _a === void 0 ? void 0 : _a.value.match(/^\s*/)) === null || _b === void 0 ? void 0 : _b[0].length) || 0; });
    const blockAnnotations = annotations.filter(isBlockAnnotation);
    const inlineAnnotations = annotations.filter(isInlineAnnotation);
    const groups = toLineGroups(lines, blockAnnotations);
    const StackedPre = useMemo(() => {
        return getPreComponent(handlers);
    }, [handlers.map((c) => c.name).join(",")]);
    return (_jsx(StackedPre, { ref: ref, "data-theme": themeName, "data-lang": lang, className: className, ...rest, children: _jsx(RenderLines, { linesOrGroups: groups, handlers: handlers, inlineAnnotations: inlineAnnotations, indentations: indentations }) }));
});
function RenderLines({ linesOrGroups, handlers, inlineAnnotations, indentations, annotationStack = [], }) {
    return linesOrGroups.map((group) => {
        if (isGroup(group)) {
            return (_jsx(AnnotatedLines, { group: group, handlers: handlers, inlineAnnotations: inlineAnnotations, annotationStack: annotationStack, indentations: indentations }, group.range[0]));
        }
        const lineNumber = group.range[0];
        const indentation = indentations[lineNumber - 1];
        const lineAnnotations = inlineAnnotations.filter((annotation) => annotation.lineNumber === lineNumber);
        const lineContent = toLineContent(group.tokens, lineAnnotations);
        let Line = getLineComponent(annotationStack, handlers);
        let children = (_jsx(RenderLineContent, { lineContent: lineContent, handlers: handlers, lineNumber: lineNumber }));
        return (_jsx(Line, { lineNumber: lineNumber, indentation: indentation, children: children }, lineNumber));
    });
}
const DefaultLine = ({ merge: base = {}, ...rest }) => {
    const { lineNumber, indentation, ...props } = mergeProps(base, rest);
    return _jsx("div", { ...props });
};
function getLineComponent(annotationStack, handlers) {
    const BaseLine = handlers.reduce((Inner, { Line }) => {
        if (!Line) {
            return Inner;
        }
        return ({ merge = {}, ...props }) => {
            const result = mergeProps(merge, props);
            return _jsx(Line, { ...result, InnerLine: Inner });
        };
    }, DefaultLine);
    return handlers.reduce((Inner, { name, AnnotatedLine }) => {
        const annotation = annotationStack.find((a) => a.name === name);
        if (!annotation || !AnnotatedLine) {
            return Inner;
        }
        return ({ merge = {}, ...props }) => {
            const result = mergeProps(merge, props);
            return (_jsx(AnnotatedLine, { ...result, annotation: annotation, InnerLine: Inner }));
        };
    }, BaseLine);
}
function AnnotatedLines({ group, handlers, inlineAnnotations, annotationStack, indentations, }) {
    var _a;
    const { annotation, lines } = group;
    const { name } = annotation;
    const Component = (_a = handlers.find((c) => c.name === name)) === null || _a === void 0 ? void 0 : _a.Block;
    if (!Component) {
        return (_jsx(RenderLines, { linesOrGroups: lines, handlers: handlers, inlineAnnotations: inlineAnnotations, annotationStack: [annotation, ...annotationStack], indentations: indentations }));
    }
    return (_jsx(Component, { annotation: annotation, children: _jsx(RenderLines, { linesOrGroups: lines, handlers: handlers, inlineAnnotations: inlineAnnotations, annotationStack: [annotation, ...annotationStack], indentations: indentations }) }));
}
function toLines(tokens) {
    const lines = [[]];
    const tokenStack = tokens.slice();
    let col = 1;
    while (tokenStack.length) {
        const token = tokenStack.shift();
        if (typeof token === "string") {
            const [value, ...tail] = token.split("\n");
            if (value) {
                let start = col;
                col += value.length;
                lines[lines.length - 1].push({
                    value,
                    range: [start, col],
                });
            }
            if (tail.length) {
                lines[lines.length - 1].push({
                    value: "\n",
                    range: [col, col + 1],
                });
                lines.push([]);
                col = 1;
                tokenStack.unshift(tail.join("\n"));
            }
        }
        else {
            const [value, color, rest = {}] = token;
            let start = col;
            col += value.length;
            lines[lines.length - 1].push({
                value,
                style: { color, ...rest },
                range: [start, col],
            });
        }
    }
    return lines.map((tokens, i) => ({ tokens, range: [i + 1, i + 1] }));
}
function toLineGroups(lines, annotations) {
    let groups = lines;
    for (let i = annotations.length - 1; i >= 0; i--) {
        const annotation = annotations[i];
        groups = applyBlockAnnotation(groups, annotation);
    }
    return groups;
}
function applyBlockAnnotation(lines, annotation) {
    const { fromLineNumber, toLineNumber } = annotation;
    const range = [fromLineNumber, toLineNumber];
    let groups = splitGroups(lines, range[0]);
    groups = splitGroups(groups, range[1] + 1);
    let before = [];
    let inside = [];
    let after = [];
    groups.forEach((group) => {
        if (group.range[1] < range[0]) {
            before.push(group);
        }
        else if (group.range[0] > range[1]) {
            after.push(group);
        }
        else {
            inside.push(group);
        }
    });
    return [
        ...before,
        {
            annotation,
            lines: inside,
            range: [inside[0].range[0], inside[inside.length - 1].range[1]],
        },
        ...after,
    ];
}
function splitGroups(groups, lineNumber) {
    const index = groups.findIndex((group) => {
        return (isGroup(group) &&
            group.range[0] < lineNumber &&
            lineNumber <= group.range[1]);
    });
    if (index === -1) {
        return groups;
    }
    const group = groups[index];
    const lines = splitGroups(group.lines, lineNumber);
    let before = [];
    let after = [];
    lines.forEach((lineOrGroup) => {
        if (lineOrGroup.range[1] < lineNumber) {
            before.push(lineOrGroup);
        }
        else {
            after.push(lineOrGroup);
        }
    });
    return [
        ...groups.slice(0, index),
        { ...group, lines: before, range: [group.range[0], lineNumber - 1] },
        { ...group, lines: after, range: [lineNumber, group.range[1]] },
        ...groups.slice(index + 1),
    ];
}
function isGroup(item) {
    return item.annotation !== undefined;
}
