import { jsx as _jsx } from "react/jsx-runtime";
import { forwardRef } from "react";
import { RenderLineContent, toLineContent } from "./tokens.js";
import { isBlockAnnotation, isInlineAnnotation, } from "./types.js";
export const CodeRender = forwardRef(({ info, components = {}, className, ...rest }, ref) => {
    const { tokens, themeName, lang, annotations } = info;
    if (!tokens) {
        throw new Error("Missing tokens in code block. Use the `highlight` function to generate the tokens.");
    }
    const lines = toLines(tokens);
    const blockAnnotations = annotations.filter(isBlockAnnotation);
    const inlineAnnotations = annotations.filter(isInlineAnnotation);
    const groups = toLineGroups(lines, blockAnnotations);
    return (_jsx("pre", { ref: ref, "data-theme": themeName, "data-lang": lang, className: className, ...rest, children: _jsx(RenderLines, { linesOrGroups: groups, components: components, inlineAnnotations: inlineAnnotations }) }));
});
function RenderLines({ linesOrGroups, components, inlineAnnotations, annotationStack = [], }) {
    return linesOrGroups.map((group) => {
        if (isGroup(group)) {
            return (_jsx(AnnotatedLines, { group: group, components: components, inlineAnnotations: inlineAnnotations, annotationStack: annotationStack }, group.range[0]));
        }
        const lineNumber = group.range[0];
        const lineAnnotations = inlineAnnotations.filter((annotation) => annotation.lineNumber === lineNumber);
        const lineContent = toLineContent(group.tokens, lineAnnotations);
        // find the first annotation that has a Line component or "Line"
        const annotation = annotationStack.find(({ name }) => components[("Line" + name)]);
        let Line = components.Line;
        let children = (_jsx(RenderLineContent, { lineContent: lineContent, components: components, lineNumber: lineNumber }));
        if (annotation) {
            let { name } = annotation;
            const LineComponent = components[("Line" + name)];
            return (_jsx(LineComponent, { lineNumber: lineNumber, annotation: annotation, children: children }));
        }
        else if (Line) {
            return (_jsx(Line, { lineNumber: lineNumber, children: children }, lineNumber));
        }
        else {
            return children;
        }
    });
}
function AnnotatedLines({ group, components, inlineAnnotations, annotationStack, }) {
    const { annotation, lines } = group;
    const { name } = annotation;
    const Component = components[("Block" + name)];
    if (!Component) {
        return (_jsx(RenderLines, { linesOrGroups: lines, components: components, inlineAnnotations: inlineAnnotations, annotationStack: [annotation, ...annotationStack] }));
    }
    return (_jsx(Component, { annotation: annotation, children: _jsx(RenderLines, { linesOrGroups: lines, components: components, inlineAnnotations: inlineAnnotations, annotationStack: [annotation, ...annotationStack] }) }));
}
function toLines(tokens) {
    const lines = [[]];
    const tokenStack = tokens.slice();
    let col = 1;
    while (tokenStack.length) {
        const token = tokenStack.shift();
        if (typeof token === "string") {
            const [value, ...tail] = token.split("\n");
            if (value) {
                let start = col;
                col += value.length;
                lines[lines.length - 1].push({
                    value,
                    range: [start, col],
                });
            }
            if (tail.length) {
                lines[lines.length - 1].push({
                    value: "\n",
                    range: [col, col + 1],
                });
                lines.push([]);
                col = 1;
                tokenStack.unshift(tail.join("\n"));
            }
        }
        else {
            const [value, color, rest = {}] = token;
            let start = col;
            col += value.length;
            lines[lines.length - 1].push({
                value,
                style: { color, ...rest },
                range: [start, col],
            });
        }
    }
    return lines.map((tokens, i) => ({ tokens, range: [i + 1, i + 1] }));
}
function toLineGroups(lines, annotations) {
    let groups = lines;
    for (let i = annotations.length - 1; i >= 0; i--) {
        const annotation = annotations[i];
        groups = applyBlockAnnotation(groups, annotation);
    }
    return groups;
}
function applyBlockAnnotation(lines, annotation) {
    const { fromLineNumber, toLineNumber } = annotation;
    const range = [fromLineNumber, toLineNumber];
    let groups = splitGroups(lines, range[0]);
    groups = splitGroups(groups, range[1] + 1);
    let before = [];
    let inside = [];
    let after = [];
    groups.forEach((group) => {
        if (group.range[1] < range[0]) {
            before.push(group);
        }
        else if (group.range[0] > range[1]) {
            after.push(group);
        }
        else {
            inside.push(group);
        }
    });
    return [
        ...before,
        {
            annotation,
            lines: inside,
            range: [inside[0].range[0], inside[inside.length - 1].range[1]],
        },
        ...after,
    ];
}
function splitGroups(groups, lineNumber) {
    const index = groups.findIndex((group) => {
        return (isGroup(group) &&
            group.range[0] < lineNumber &&
            lineNumber <= group.range[1]);
    });
    if (index === -1) {
        return groups;
    }
    const group = groups[index];
    const lines = splitGroups(group.lines, lineNumber);
    let before = [];
    let after = [];
    lines.forEach((lineOrGroup) => {
        if (lineOrGroup.range[1] < lineNumber) {
            before.push(lineOrGroup);
        }
        else {
            after.push(lineOrGroup);
        }
    });
    return [
        ...groups.slice(0, index),
        { ...group, lines: before, range: [group.range[0], lineNumber - 1] },
        { ...group, lines: after, range: [lineNumber, group.range[1]] },
        ...groups.slice(index + 1),
    ];
}
function isGroup(item) {
    return item.annotation !== undefined;
}
